package Luokat;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import Luokat.Reissu;

/**
 * Pitää sisällään kaikki reissut
 */
public class Reissut {

	    private List<Reissu> reissut;

	/**
	 * muodostaja
	 */
	public Reissut(Tulkki lajit, Tulkki koot, Tulkki vieheet) {
		this.reissut = new ArrayList<>();

		luoReissut(lajit, koot,vieheet);
	}

	private List<Saalis> luoSaaliit(Tulkki lajit, Tulkki koot, Tulkki vieheet) {
		TiedostonHiplailija t = new TiedostonHiplailija(System.getProperty("user.dir") + "/../saaliit.dat");
		List<Saalis> saaliit = new ArrayList<Saalis>();
		List<String> tiedot = t.lueRivit();
		t.sulje();
		for (String s : tiedot) {
			Saalis saalis = new Saalis(s, lajit, vieheet, koot);
			saaliit.add(saalis);
		}
		return saaliit;
	}
	
	private void luoReissut(Tulkki lajit, Tulkki koot, Tulkki vieheet) {
		TiedostonHiplailija t = new TiedostonHiplailija(System.getProperty("user.dir") + "/../reissut.dat");
		List<Saalis> saaliit = luoSaaliit(lajit, koot, vieheet); //Kaikki saaliit
		for (String s : t.lueRivit()) {
			Reissu a = new Reissu(s);
			List<Saalis> apuri = new ArrayList<>(); // Tässä luodaan tietyn reissun saaliiden lista
			for (Saalis saalis : saaliit) {
				if (saalis.getReissunTagi() == a.getTag()) {
					apuri.add(saalis);
				}
				if (saalis.getReissunTagi() > a.getTag()) {
					break;
				}
			}
			a.setSaaliit(apuri);
			this.reissut.add(a);
		}
	}

	/**
	 * lisää reissun listaan
	 * @param reissu mikä reissu lisätään
	 */
	public void lisaaListaan(Reissu reissu) {
	   	reissut.add(reissu);
	}


	/**
	 * palauttaa reissu listan
	 * @return reissu lista
	 */
	public List<Reissu> getReissut() {
		return reissut;
	}
	
	
	

	/**
	 * etsii reissun päivämäärän perusteella
	 * @param paivamaara mikä päivä
	 * @return em. päivän reissun
	 */
	
	public static Reissu etsi(String paivamaara, Reissut reissut) {
		
		for (Reissu t : reissut.getReissut()) {
			if (t.getPaivamaara().equals(paivamaara)) {
				return t;
			}
		}
		return null;
	}
	
	public void sort() {
		
		reissut.sort(Comparator.comparing(Reissu::getPaivamaara).reversed());
		
	}
	
	
	public void removeReissu() {
		   
		   Iterator<Reissu> iterator = reissut.iterator();
		    while (iterator.hasNext()) {
		        Reissu reissu = iterator.next();
		        if (reissu.getpoistetaanko()) {
		            iterator.remove();  
		            break;
		        }
		    }
	}

	    // Testi
	    public static void main(String[] args) {
			Tulkki lajit = new Tulkki(System.getProperty("user.dir") + "/../lajit.dat");
			Tulkki vieheTulkki = new Tulkki(System.getProperty("user.dir") + "/../vieheTyypit.dat");
			Tulkki kokoTulkki = new Tulkki(System.getProperty("user.dir") + "/../koot.dat");

			Reissut reissut = new Reissut(lajit, kokoTulkki, vieheTulkki);
			Reissu t0 = new Reissu( Reissu.sToDate("1.12.2323"), "ka", "pilvi", new ArrayList<>(), false);
	        Reissu t1 = new Reissu( Reissu.sToDate("12.12.2023"), "jk", "sade" , new ArrayList<>(), false);
	        Reissu t2 = new Reissu( Reissu.sToDate("12.12.1223"), "sk", "aurinko", new ArrayList<>(), false );
	        Reissu t3 = new Reissu( Reissu.sToDate("12.12.2323"), "ka", "pilvi", new ArrayList<>(), false);

	        reissut.lisaaListaan(t1); reissut.lisaaListaan(t2); reissut.lisaaListaan(t3); reissut.lisaaListaan(t0);
	        
	        for (Reissu t: reissut.getReissut()) {
        		System.out.println("ennen " + t.getPvmString());
        	}
	        
	        reissut.sort();
	        	
	        	for (Reissu t: reissut.getReissut()) {
	        		System.out.println("jälkeen " + t.getPvmString());
	        	}
	        	
	       
	    }
	//todo:
	//tallenna()
	    

	    // kesken...
	public int[] paivamaaraToInt(String pm) {
		String[] osat = pm.split(".");
		if (osat.length < 3) {
		return null;
		}
		int[] pv = new int[3];
		
		pv[0] = Integer.parseInt(osat[0]);
		pv[1] = Integer.parseInt(osat[1]);
		pv[2] = Integer.parseInt(osat[2]);
		return pv;
	
	}
//todo:
// tallenna()
}
